# File t_fibarea.x
# By Doug Mink, after a shell script by Brian McLeod
# December 13, 2006

include	<mach.h>
include	<imhdr.h>
include <imio.h>
include	<math.h>
include <imset.h>


# T_FIBAREA -- Compute and print the statistics of images.

procedure t_fibarea ()

pointer	im
pointer	immap(), list
char	imname[SZ_FNAME]
double	area
int	imtopenp(), imtgetim()
bool	clgetb()
errchk	immap()
bool	whead, verbose, debug

begin

	# Open the list of input images
	list = imtopenp ("images")
	whead = clgetb ("whead")
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")
	if (debug) {
	    verbose = TRUE
	    call printf ("filename	pra	pdec	ra	dec	deg	mm	radial	tangent	area\n");
	    call printf ("--------	---	----	--	---	---	--	------	-------	----\n");
	    }
	else if (verbose) {
	    call printf ("filename	deg	mm	radial	tangent	area\n");
	    call printf ("--------	---	--	------	-------	----\n");
	    }

	# Loop through the input images.
	im = NULL
	while (imtgetim (list, imname, SZ_FNAME) != EOF) {

	    # Open the image.
	    iferr (im = immap (imname, READ_WRITE, 0)) {
		call printf ("Error reading image %s ...\n")
		    call pargstr (imname)
		next
		}
	    if (im == NULL)
		next

	    # Set fiber area as FIBAREA in spectrum header
	    IM_UPDATE(im) = NO
	    call fib_setarea (im, imname, area, whead, verbose, debug)
	    if (!whead) {
		call clputd ("area", area)
		}

	    # Unmap the image
	    call imunmap (im)
	    im = NULL
	    }

	call imtclose (list)
end


# FIB_SETAREA -- Set fiber area as FIBAREA in spectrum header

procedure fib_setarea (im, imname, area, whead, verbose, debug)

pointer	im		# Image data structure
char	imname[ARB]	# Image file name
double	area		# Area of fiber on the sky (returned)
bool	whead		# If true, add fiber area as FIBAREA to header
bool	verbose		# If true, print table of areas
bool	debug		# If true, print table of positions

double	ra, dindef, dec, pra, pdec
double	angle
char	str[SZ_LINE]
char	colon
int	stridx()

begin
	dindef = INDEFD
	colon = ':'

# Direction of observed object (Convert RA to degrees if sexigesimal)
	ra = dindef
	call imgdpar (im, "RA", ra)
	if (ra != dindef) {
	    call imgspar (im, "RA", str, SZ_LINE)
	    if (stridx (colon, str) != 0)
		ra = ra * 15.0
	    }
	dec = dindef
	call imgdpar (im, "DEC", dec)

	if (ra == dindef || dec == dindef) {
	    call printf ("*** No RA or DEC for object in image header\n");
	    return
	    }

# Direction of plate center (Convert RA to degrees if sexigesimal)
	pra = dindef
	call imgdpar (im, "PRA", pra)
	if (pra != dindef) {
	    call imgspar (im, "PRA", str, SZ_LINE)
	    if (stridx (colon, str) != 0)
		pra = pra * 15.0
	    }
	if (pra == dindef) {
	    call imgdpar (im,"CAT-RA",pra)
	    if (pra != dindef) {
		call imgspar (im, "CAT-RA", str, SZ_LINE)
		if (stridx (colon, str) != 0)
		    pra = pra * 15.0
		}
	    }
	pdec = dindef
	call imgdpar (im,"PDEC",pdec)
	if (pdec == dindef)
	    call imgdpar (im,"CAT-DEC",pdec)

	if (pra == dindef || pdec == dindef) {
	    call printf ("*** No pointing center in image header\n");
	    return
	    }

	if (verbose || debug)
            call printf ("%32s	")
		call pargstr (imname)

	if (debug) {
            call printf ("%9.6f	%9.6f	%9.6f	%9.6f	")
		call pargd (pra)
		call pargd (pdec)
		call pargd (ra)
		call pargd (dec)
	    }

# Compute angle from optical axis (pointing direction) to fiber
	call fib_diff (pra, pdec, ra, dec, angle, verbose)

# Compute area of fiber in square arcseconds
	call fib_comparea (imname, verbose, whead, angle, area)

# Write area subtended by the fiber to the spectrum header
	if (whead) {
	    call imaddd (im,"FIBAREA",area)
	    IM_UPDATE(im) = YES
	    }
	return
end


# FIB_COMPAREA -- Compute fiber area from field and fiber RA, DEC

procedure fib_comparea (imname, verbose, whead, angle, area)

char	imname[ARB]	# Name of spectrum file
bool	verbose		# If true, print table of results
bool	whead		# True if area being written to header
double  angle		# Angle on sky from optical axis in degrees
double  area		# Area of sky in this fiber in square arcseconds

double	r, radial, tangential
double	deg, deg2, deg3, deg4, deg5

begin
	deg = angle
	deg2 = deg * deg
	deg3 = deg2 * deg
	deg4 = deg3 * deg
	deg5 = deg4 * deg

	r = 599.9472d0 * deg + 37.369d0 * deg3 + 28.0281d0 * deg5

	radial = (599.9472d0 + 3.d0*37.369d0*deg2 + 5.d0*28.0281d0*deg4) / 3600.d0;

	if ( deg == 0 )
            tangential = radial
	else
            tangential = r / deg / 3600.d0

	area = 3.1415926535d0 / radial / tangential * 0.125d0 * 0.125d0;

	if (verbose) {
            call printf ("%7.3f	%7.3f	%8.6f	%8.6f	%8.6f")
		call pargd (deg)
		call pargd (r)
		call pargd (radial)
		call pargd (tangential)
		call pargd (area)
	    if (whead)
		call printf (" -> FIBAREA")
	    call printf ("\n")
	    }
end


# FIB_DIFF -- Compute distance in degrees between two sky coordinates

procedure fib_diff (ra1,dec1,ra2,dec2, angle)

double ra1, dec1	# Right ascension and declination in degrees
double ra2, dec2	# Right ascension and declination in degrees
double angle		# Separation angle in degrees (returned)

double xr1, xr2, yr1, yr2, pi, deg2rad, rad2deg
double pos1[3], pos2[3], w, diff, cosb
int i

begin
	pi =  3.141592653589793238462643d0
	deg2rad = pi / 180.d0
	rad2deg = 180.d0 / pi

#  Convert two vectors to direction cosines
	xr1 = ra1 * deg2rad
	yr1 = dec1 * deg2rad
	cosb = cos (yr1)
	pos1[0] = cos (xr1) * cosb
	pos1[1] = sin (xr1) * cosb
	pos1[2] = sin (yr1)

	xr2 = ra2 * deg2rad
	yr2 = dec2 * deg2rad
	cosb = cos (yr2)
	pos2[0] = cos (xr2) * cosb
	pos2[1] = sin (xr2) * cosb
	pos2[2] = sin (yr2)

# Modulus squared of half the difference vector
	w = 0.0
	for (i = 0; i < 3; i = i + 1) {
	    w = w + (pos1[i] - pos2[i]) * (pos1[i] - pos2[i])
	    }
	w = w / 4.0
	if (w > 1.0) w = 1.0

# Angle beween the vectors
	diff = 2.0 * atan2 (sqrt (w), sqrt (1.0 - w))
	angle = diff * rad2deg
	return

end
